@@ -678,7 +678,7 @@ def sql_name_pattern(pattern):
 def show_help(cur, arg, verbose):  # All the parameters are ignored.
     headers = ['Command', 'Description']
     result = []
-    for command, value in CASE_SENSITIVE_COMMANDS.iteritems():
+    for command, value in sorted(CASE_SENSITIVE_COMMANDS.iteritems()):
         if value[1]:
             result.append(value[1])
     return [(result, headers, None)]
@@ -702,6 +702,30 @@ CASE_SENSITIVE_COMMANDS = {
             'pg_catalog' AND n.nspname <> 'information_schema' AND n.nspname !~
             '^pg_toast' AND pg_catalog.pg_table_is_visible(c.oid) ORDER BY
             1,2;''', ['\dt', 'list tables.']),
+            '\di': ('''SELECT n.nspname as "Schema", c.relname as "Name", CASE
+            c.relkind WHEN 'r' THEN 'table' WHEN 'v' THEN 'view' WHEN 'm' THEN
+            'materialized view' WHEN 'i' THEN 'index' WHEN 'S' THEN 'sequence'
+            WHEN 's' THEN 'special' WHEN 'f' THEN 'foreign table' END as
+            "Type", pg_catalog.pg_get_userbyid(c.relowner) as "Owner",
+            c2.relname as "Table" FROM pg_catalog.pg_class c LEFT JOIN
+            pg_catalog.pg_namespace n ON n.oid = c.relnamespace LEFT JOIN
+            pg_catalog.pg_index i ON i.indexrelid = c.oid LEFT JOIN
+            pg_catalog.pg_class c2 ON i.indrelid = c2.oid WHERE c.relkind
+            IN ('i','') AND n.nspname <> 'pg_catalog' AND
+            n.nspname <> 'information_schema' AND n.nspname !~ '^pg_toast'
+            AND pg_catalog.pg_table_is_visible(c.oid)
+            ORDER BY 1,2;''', ['\di', 'list indexes.']),
+            '\dv': ('''SELECT n.nspname as "Schema", c.relname as "Name", CASE
+            c.relkind WHEN 'r' THEN 'table' WHEN 'v' THEN 'view' WHEN 'm' THEN
+            'materialized view' WHEN 'i' THEN 'index' WHEN 'S' THEN 'sequence'
+            WHEN 's' THEN 'special' WHEN 'f' THEN 'foreign table' END as
+            "Type", pg_catalog.pg_get_userbyid(c.relowner) as "Owner" FROM
+            pg_catalog.pg_class c LEFT JOIN pg_catalog.pg_namespace n ON
+            n.oid = c.relnamespace WHERE c.relkind IN ('v','') AND
+            n.nspname <> 'pg_catalog' AND n.nspname <> 'information_schema'
+            AND n.nspname !~ '^pg_toast' AND
+            pg_catalog.pg_table_is_visible(c.oid) ORDER BY 1,2;''',
+            ['\dv', 'list views.']),
             }
 
 NON_CASE_SENSITIVE_COMMANDS = {
