@@ -1,27 +1,14 @@
+import logging
 import psycopg2
 from .packages import pgspecial
 
+_logger = logging.getLogger(__name__)
+
 def _parse_dsn(dsn, default_user, default_password, default_host,
         default_port):
     """
     This function parses a postgres url to get the different components.
 
-    >>> _parse_dsn('postgres://user:password@host:5432/dbname', 'fuser', 'fpasswd', 'fhost', '1234')
-    ('dbname', 'user', 'password', 'host', '5432')
-    >>> _parse_dsn('postgres://user@host:5432/dbname', 'fuser', 'fpasswd', 'fhost', '1234')
-    ('dbname', 'user', 'fpasswd', 'host', '5432')
-    >>> _parse_dsn('postgres://localhost:5432/dbname', 'fuser', 'fpasswd', 'fhost', '1234')
-    ('dbname', 'fuser', 'fpasswd', 'localhost', '5432')
-    >>> _parse_dsn('postgres://user:password@host/dbname', 'fuser', 'fpasswd', 'fhost', '1234')
-    ('dbname', 'user', 'password', 'host', '1234')
-    >>> _parse_dsn('postgres://user@host/dbname', 'fuser', 'fpasswd', 'fhost', '1234')
-    ('dbname', 'user', 'fpasswd', 'host', '1234')
-    >>> _parse_dsn('postgres://localhost/dbname', 'fuser', 'fpasswd', 'fhost', '1234')
-    ('dbname', 'fuser', 'fpasswd', 'localhost', '1234')
-    >>> _parse_dsn('postgres:///dbname', 'fuser', 'fpasswd', 'fhost', '1234')
-    ('dbname', 'fuser', 'fpasswd', 'fhost', '1234')
-    >>> _parse_dsn('postgresql://user:password@host:5432/dbname', 'fuser', 'fpasswd', 'fhost', '1234')
-    ('dbname', 'user', 'password', 'host', '5432')
     """
 
     user = password = host = port = dbname = None
@@ -46,6 +33,10 @@ def _parse_dsn(dsn, default_user, default_password, default_host,
     port = port or default_port
     dbname = dbname or dsn
 
+    _logger.debug('Parsed connection params:'
+            'dbname: %r, user: %r, password: %r, host: %r, port: %r',
+            dbname, user, password, host, port)
+
     return (dbname, user, password, host, port)
 
 class PGExecute(object):
@@ -59,7 +50,14 @@ class PGExecute(object):
     columns_query = '''SELECT column_name FROM information_schema.columns WHERE
     table_name =%s;'''
 
-    databases_query = '''SELECT datname FROM pg_database;'''
+    databases_query = """SELECT d.datname as "Name",
+       pg_catalog.pg_get_userbyid(d.datdba) as "Owner",
+       pg_catalog.pg_encoding_to_char(d.encoding) as "Encoding",
+       d.datcollate as "Collate",
+       d.datctype as "Ctype",
+       pg_catalog.array_to_string(d.datacl, E'\n') AS "Access privileges"
+    FROM pg_catalog.pg_database d
+    ORDER BY 1;"""
 
     def __init__(self, database, user, password, host, port):
         (self.dbname, self.user, self.password, self.host, self.port) = \
@@ -86,22 +84,27 @@ class PGExecute(object):
         # that cannot be offloaded to `pgspecial` lib. Because we have to
         # change the database connection that we're connected to.
         if sql.startswith('\c') or sql.lower().startswith('use'):
+            _logger.debug('Database change command detected.')
             try:
                 dbname = sql.split()[1]
             except:
+                _logger.debug('Failed to switch. DB missing: %r', dbname)
                 raise RuntimeError('Database name missing.')
             self.conn = psycopg2.connect(database=dbname,
                     user=self.user, password=self.password, host=self.host,
                     port=self.port)
             self.dbname = dbname
             self.conn.autocommit = True
+            _logger.debug('Successfully switched to DB: %r', dbname)
             return [(None, None, 'You are now connected to database "%s" as '
                     'user "%s"' % (self.dbname, self.user))]
 
         with self.conn.cursor() as cur:
             try:
+                _logger.debug('Trying a pgspecial command. sql: %r', sql)
                 return pgspecial.execute(cur, sql)
             except KeyError:
+                _logger.debug('Regular sql statement. sql: %r', sql)
                 cur.execute(sql)
 
             # cur.description will be None for operations that do not return
@@ -110,6 +113,7 @@ class PGExecute(object):
                 headers = [x[0] for x in cur.description]
                 return [(cur.fetchall(), headers, cur.statusmessage)]
             else:
+                _logger.debug('No rows in result.')
                 return [(None, None, cur.statusmessage)]
 
     def tables(self):
